//   Copyright 2020 Esri
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//   https://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.


#include "Display_a_map.h"
#include "ArcGISRuntimeEnvironment.h"
#include "Map.h"
#include "MapQuickView.h"

#include "SimpleRenderer.h"
#include "SimpleMarkerSymbol.h"
#include "TextSymbol.h"
#include "LocatorTask.h"
#include "SuggestResult.h"
#include "SuggestListModel.h"

#include <QUrl>

#include <QAbstractListModel>
#include <QGeoPositionInfoSource>

using namespace Esri::ArcGISRuntime;

Display_a_map::Display_a_map(QObject* parent /* = nullptr */):
  QObject(parent),
  m_map(new Map(BasemapStyle::ArcGISTopographic, this))
{

  setupLocatorTask();

}
Display_a_map::~Display_a_map()
{
}

void Display_a_map::setupLocatorTask()
{
  const QUrl geocode_service("https://geocode-api.arcgis.com/arcgis/rest/services/World/GeocodeServer");
  m_locatorTask = new LocatorTask(geocode_service, this);

  SuggestParameters suggestParams;
  const QStringList categories{"Address", "POI", "Populated Place"};
  suggestParams.setCategories(categories);
  suggestParams.setMaxResults(5);
  m_locatorTask->suggestions()->setSuggestParameters(suggestParams);

  m_geocodeParams.setMinScore(75);
  m_geocodeParams.setResultAttributeNames(QStringList {"Place_addr", "Match_addr"});
  m_suggestListModel = m_locatorTask->suggestions();
  emit suggestionsChanged();

  // connect to geocode complete signal on the LocatorTask
  connect(m_locatorTask, &LocatorTask::geocodeCompleted, this, [this](QUuid, const QList<GeocodeResult>& geocodeResults)
  {
    if (!geocodeResults.isEmpty() && m_graphicResultLocation)
    {
      // display geocode result label and position
      const GeocodeResult geocodeResult = geocodeResults.at(0);
      m_graphicResultLocation->setGeometry(geocodeResult.displayLocation());
      m_graphicResultLocation->attributes()->setAttributesMap(geocodeResult.attributes());
      constexpr double scale = 8000.0;
      m_mapView->setViewpointCenter(geocodeResult.extent().center(), scale);
      m_graphicResultLocation->setVisible(true);

      m_textSymbol->setText(geocodeResult.label());
      m_graphicResultText->setGeometry(geocodeResult.displayLocation());
      m_graphicResultText->attributes()->setAttributesMap(geocodeResult.attributes());
      m_graphicResultLocation->setVisible(true);
    }
  });
}

MapQuickView* Display_a_map::mapView() const
{
  return m_mapView;
}

// modified setupViewpoint to handle mouseevent

void Display_a_map::setupViewpoint()
{

  connect(m_mapView, &MapQuickView::mousePressed, this, [this](QMouseEvent& /* event */)
  {
    emit hideSuggestionView();
  });

  const Point center(-118.80543, 34.02700, SpatialReference::wgs84());
  const Viewpoint viewpoint(center, 100000.0);
  m_mapView->setViewpoint(viewpoint);

  configureGraphic();

}

// Set the view (created in QML)
void Display_a_map::setMapView(MapQuickView* mapView)
{
  if (!mapView || mapView == m_mapView)
  {
    return;
  }

  m_mapView = mapView;
  m_mapView->setMap(m_map);

  setupViewpoint();
  emit mapViewChanged();
}

void Display_a_map::setSuggestions(const QString& text)
{
  if (!m_suggestListModel)
    return;

  SuggestListModel* suggestListModel = dynamic_cast<SuggestListModel*>(m_suggestListModel);

  if (!suggestListModel)
    return;

  suggestListModel->setSearchText(text);
}

void Display_a_map::configureGraphic()
{
  if (m_graphicResultLocation)
    return;

  // create graphics overlay and add to map view
  m_graphicsOverlay = new GraphicsOverlay(this);

  // set a renderer on the graphics overlay
  SimpleRenderer* simpleRenderer = new SimpleRenderer(this);

  // Create a graphic and symbol to display the result location.
  SimpleMarkerSymbol* simpleMarkerSymbol = new SimpleMarkerSymbol(SimpleMarkerSymbolStyle::Square, QColor("blue"), 12.0, this);
  simpleRenderer->setSymbol(simpleMarkerSymbol);
  m_graphicResultLocation = new Graphic(this);
  m_graphicResultLocation->setSymbol(simpleMarkerSymbol);
  m_graphicsOverlay->graphics()->append(m_graphicResultLocation);

  // Create a graphic and symbol to display a label next to the result location
  m_textSymbol = new TextSymbol("", QColor("red"), 18.0, HorizontalAlignment::Center, VerticalAlignment::Bottom, this);
  m_graphicResultText = new Graphic(this);
  m_graphicResultText->setSymbol(m_textSymbol);

  m_graphicsOverlay->graphics()->append(m_graphicResultText);

  m_mapView->graphicsOverlays()->append(m_graphicsOverlay);
}

void Display_a_map::geocode(const QString& query)
{
  m_locatorTask->geocodeWithParameters(query, m_geocodeParams);
}

QAbstractListModel* Display_a_map::suggestions() const
{
  return m_suggestListModel;
}

void Display_a_map::clearGraphics()
{
  m_graphicResultLocation->setGeometry(Point());
  m_graphicResultText->setGeometry(Point());
}


